@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2020, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "test.h"
 
@@ -64,8 +66,7 @@ static const struct testparams params[] = {
   { F_RESUME |             F_FAIL | F_CONTENTRANGE,                CURLE_OK },
   { F_RESUME | F_HTTP416 |                           F_IGNOREBODY, CURLE_OK },
   { F_RESUME | F_HTTP416 |          F_CONTENTRANGE | F_IGNOREBODY, CURLE_OK },
-  { F_RESUME | F_HTTP416 | F_FAIL |                  F_IGNOREBODY,
-                                                  CURLE_HTTP_RETURNED_ERROR },
+  { F_RESUME | F_HTTP416 | F_FAIL |                  F_IGNOREBODY, CURLE_OK },
   { F_RESUME | F_HTTP416 | F_FAIL | F_CONTENTRANGE | F_IGNOREBODY,
                                                   CURLE_HTTP_RETURNED_ERROR }
 };
@@ -82,7 +83,8 @@ static size_t writedata(char *data, size_t size, size_t nmemb, void *userdata)
   return size * nmemb;
 }
 
-static int onetest(CURL *curl, const char *url, const struct testparams *p)
+static int onetest(CURL *curl, const char *url, const struct testparams *p,
+                   size_t num)
 {
   CURLcode res;
   unsigned int replyselector;
@@ -93,6 +95,7 @@ static int onetest(CURL *curl, const char *url, const struct testparams *p)
     replyselector += 2;
   msnprintf(urlbuf, sizeof(urlbuf), "%s%04u", url, replyselector);
   test_setopt(curl, CURLOPT_URL, urlbuf);
+  test_setopt(curl, CURLOPT_VERBOSE, 1L);
   test_setopt(curl, CURLOPT_RESUME_FROM, (p->flags & F_RESUME)? 3: 0);
   test_setopt(curl, CURLOPT_RANGE, !(p->flags & F_RESUME)?
                                    "3-1000000": (char *) NULL);
@@ -100,22 +103,22 @@ static int onetest(CURL *curl, const char *url, const struct testparams *p)
   hasbody = 0;
   res = curl_easy_perform(curl);
   if(res != p->result) {
-    fprintf(stderr, "bad error code (%d): resume=%s, fail=%s, http416=%s, "
-                    "content-range=%s, expected=%d\n", res,
-                    (p->flags & F_RESUME)? "yes": "no",
-                    (p->flags & F_FAIL)? "yes": "no",
-                    (p->flags & F_HTTP416)? "yes": "no",
-                    (p->flags & F_CONTENTRANGE)? "yes": "no",
-                    p->result);
+    printf("%d: bad error code (%d): resume=%s, fail=%s, http416=%s, "
+           "content-range=%s, expected=%d\n", num, res,
+           (p->flags & F_RESUME)? "yes": "no",
+           (p->flags & F_FAIL)? "yes": "no",
+           (p->flags & F_HTTP416)? "yes": "no",
+           (p->flags & F_CONTENTRANGE)? "yes": "no",
+           p->result);
     return 1;
   }
   if(hasbody && (p->flags & F_IGNOREBODY)) {
-    fprintf(stderr, "body should be ignored and is not: resume=%s, fail=%s, "
-                    "http416=%s, content-range=%s\n",
-                    (p->flags & F_RESUME)? "yes": "no",
-                    (p->flags & F_FAIL)? "yes": "no",
-                    (p->flags & F_HTTP416)? "yes": "no",
-                    (p->flags & F_CONTENTRANGE)? "yes": "no");
+    printf("body should be ignored and is not: resume=%s, fail=%s, "
+           "http416=%s, content-range=%s\n",
+           (p->flags & F_RESUME)? "yes": "no",
+           (p->flags & F_FAIL)? "yes": "no",
+           (p->flags & F_HTTP416)? "yes": "no",
+           (p->flags & F_CONTENTRANGE)? "yes": "no");
     return 1;
   }
   return 0;
@@ -125,6 +128,9 @@ static int onetest(CURL *curl, const char *url, const struct testparams *p)
   return 1;
 }
 
+/* for debugging: */
+/* #define SINGLETEST 9 */
+
 int test(char *URL)
 {
   CURLcode res;
@@ -137,20 +143,25 @@ int test(char *URL)
     return TEST_ERR_MAJOR_BAD;
   }
 
-  curl = curl_easy_init();
-  if(!curl) {
-    fprintf(stderr, "curl_easy_init() failed\n");
-    curl_global_cleanup();
-    return TEST_ERR_MAJOR_BAD;
+  for(i = 0; i < sizeof(params) / sizeof(params[0]); i++) {
+    curl = curl_easy_init();
+    if(!curl) {
+      fprintf(stderr, "curl_easy_init() failed\n");
+      curl_global_cleanup();
+      return TEST_ERR_MAJOR_BAD;
+    }
+
+    test_setopt(curl, CURLOPT_WRITEFUNCTION, writedata);
+
+#ifdef SINGLETEST
+    if(SINGLETEST == i)
+#endif
+      status |= onetest(curl, URL, params + i, i);
+    curl_easy_cleanup(curl);
   }
 
-  test_setopt(curl, CURLOPT_WRITEFUNCTION, writedata);
-
-  for(i = 0; i < sizeof(params) / sizeof(params[0]); i++)
-    status |= onetest(curl, URL, params + i);
-
-  curl_easy_cleanup(curl);
   curl_global_cleanup();
+  printf("%d\n", status);
   return status;
 
   test_cleanup:
