@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2019 - 2020, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,10 +18,13 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "test.h"
 
 #include "testutil.h"
+#include "timediff.h"
 #include "warnless.h"
 #include "memdebug.h"
 
@@ -30,10 +33,15 @@ int test(char *URL)
   CURLSH *sh = NULL;
   CURL *ch = NULL;
   int unfinished;
+  CURLM *cm;
 
-  CURLM *cm = curl_multi_init();
-  if(!cm)
+  curl_global_init(CURL_GLOBAL_ALL);
+
+  cm = curl_multi_init();
+  if(!cm) {
+    curl_global_cleanup();
     return 1;
+  }
   sh = curl_share_init();
   if(!sh)
     goto cleanup;
@@ -68,8 +76,7 @@ int test(char *URL)
     curl_multi_timeout(cm, &max_tout);
 
     if(max_tout > 0) {
-      timeout.tv_sec = max_tout / 1000;
-      timeout.tv_usec = (max_tout % 1000) * 1000;
+      curlx_mstotv(&timeout, max_tout);
     }
     else {
       timeout.tv_sec = 0;
