@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "test.h"
 
@@ -34,7 +36,6 @@
 
 #if !defined(HAVE_POLL_FINE)    && \
     !defined(USE_WINSOCK)       && \
-    !defined(TPF)               && \
     !defined(FD_SETSIZE)
 #error "this test requires FD_SETSIZE"
 #endif
@@ -382,9 +383,7 @@ static int rlimit(int keep_open)
   msnprintf(strbuff, sizeof(strbuff), fmt, num_open.rlim_max);
   fprintf(stderr, "%s file descriptors open\n", strbuff);
 
-#if !defined(HAVE_POLL_FINE)    && \
-    !defined(USE_WINSOCK)       && \
-    !defined(TPF)
+#if !defined(HAVE_POLL_FINE) && !defined(USE_WINSOCK)
 
   /*
    * when using select() instead of poll() we cannot test
